import pytest
from evmosgrpc.messages.msgsend import create_msg_send

def test_create_msg_send():
    a = create_msg_send("evmos1h8057cmr2znknsaqu6vqazfughcfq93k24dmw4","evmos1dlt6lgs6sslurfl3zyl7tgmnvzhq5us2uquxv7", 100, "evmos") # NOQA E501
    assert a.SerializeToString() == b'\n,evmos1h8057cmr2znknsaqu6vqazfughcfq93k24dmw4\x12,evmos1dlt6lgs6sslurfl3zyl7tgmnvzhq5us2uquxv7\x1a\x0c\n\x05evmos\x12\x03100' # NOQA E501



from evmoswallet import Wallet
from evmosgrpc.accounts import get_account_grpc
from google.protobuf.message import Message
from evmosgrpc.broadcaster import broadcast


class TransactionBuilder():
    def __init__(self, seed: str) -> None:
        self.wallet = Wallet(seed)
        self.address = self.wallet.evmos_address
        self.account_number, self.sequence = get_account_grpc(self.address)

    def send_tx(self, msg: Message):
        self.update_sequence()
        return broadcast(msg)

    def update_sequence(self):
        _, self.sequence = get_account_grpc(self.address)



from google.protobuf.json_format import MessageToDict

from evmosproto.cosmos.auth.v1beta1.query_pb2 import QueryAccountRequest
from evmosproto.cosmos.auth.v1beta1.query_pb2_grpc import QueryStub
from evmosproto.ethermint.types.v1.account_pb2 import EthAccount
from typing import Union
from evmosgrpc.utils import create_grpc_channel


# Get account, returns pub_key, account_number, sequence
def get_account_grpc(account: str) -> Union[tuple[str, str], tuple[None, None]]:
    # Create channel
    stub = QueryStub(create_grpc_channel())
    params = QueryAccountRequest()
    params.address = account
    # Send the request
    resp = stub.Account(params)
    # Parse the Any message
    account = EthAccount()
    account.ParseFromString(resp.account.value)
    account = MessageToDict(account, including_default_value_fields=True)['baseAccount']
    return account['accountNumber'], account['sequence']
